package com.example.chatclient;

import android.widget.TextView;

import org.w3c.dom.Text;

import java.io.DataInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Socket;

public class Client extends Thread {

    WriteThread writeThread;
    Callback callback;

    public void send(String msg) {
        writeThread.setMessage(msg);
    }

    public Client(Callback callback) {
        this.callback = callback;
    }

    @Override
    public void run() {
        Socket socket = null;
        try {
            socket = new Socket("192.168.0.2", 9125);

            ReadThread readThread = new ReadThread(socket, callback);
            writeThread = new WriteThread(socket);

            readThread.start();
            writeThread.start();

            readThread.join();
            writeThread.join();

        } catch (IOException | InterruptedException e) {
            e.printStackTrace();
            if (socket != null) {
                try {
                    socket.close();
                } catch (IOException exception) {
                    exception.printStackTrace();
                }
            }
        }
    }
}

class ReadThread extends Thread {
    private Socket socket;

    private Callback callback;

    public ReadThread(Socket socket, Callback callback) {
        this.socket = socket;
        this.callback = callback;
    }

    @Override
    public void run() {
        while (!isInterrupted()) {
            try {
                ObjectInputStream ois = new ObjectInputStream(socket.getInputStream());

                Transport t = (Transport) ois.readObject();

                System.out.print("\033[1;31m\033[40m " + t.Name + ": \u001B[0m ");
                System.out.println(t.Message);

//                textView.setText(t.Name + ": " + t.Message);
                callback.RecvMessage(t.Message);

            } catch (IOException | ClassNotFoundException exception) {
                exception.printStackTrace();
                try {
                    socket.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                return;
            }
        }
    }
}

class WriteThread extends Thread {
    private Socket socket;
    private final String Name = "Alec";
    private boolean isMessage = false;
    private String message;

    public WriteThread(Socket socket) {
        this.socket = socket;
    }

    public void setMessage(String message) {
        isMessage = true;
        this.message = message;
    }

    @Override
    public void run() {
        while (!isInterrupted()) {
            if (!isMessage) {
                continue;
            }
            isMessage = false;
            try {
                ObjectOutputStream ois = new ObjectOutputStream(socket.getOutputStream());

                Transport t = new Transport();
                t.Name = this.Name;
                t.Message = message;

                ois.writeObject(t);
                ois.flush();

            } catch (IOException exception) {
                exception.printStackTrace();
                try {
                    socket.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                return;
            }
        }
    }
}
